#! /bin/sh
#
# Copyright by The HDF Group.
# Copyright by the Board of Trustees of the University of Illinois.
# All rights reserved.
#
# This file is part of HDF5.  The full HDF5 copyright notice, including
# terms governing use, modification, and redistribution, is contained in
# the files COPYING and Copyright.html.  COPYING can be found at the root
# of the source code distribution tree; Copyright.html can be found at the
# root level of an installed copy of the electronic HDF5 document set and
# is linked from the top-level documents page.  It can also be found at
# http://hdfgroup.org/HDF5/doc/Copyright.html.  If you do not have
# access to either file, you may request a copy from help@hdfgroup.org.
#
# Tests for the h5dump tool

srcdir=.

TESTNAME=h5dumpxml
EXIT_SUCCESS=0
EXIT_FAILURE=1

H5_LONE_COLON="yes"

DUMPER=h5dump               # The tool name
DUMPER_BIN=`pwd`/$DUMPER    # The path of the tool binary

RM='rm -rf'
CMP='cmp -s'
DIFF='diff -c'
CP='cp'
DIRNAME='dirname'
LS='ls'
AWK='awk'

nerrors=0
verbose=yes

# source dirs
SRC_TOOLS="$srcdir/../"

SRC_TOOLS_TESTFILES="$SRC_TOOLS/testfiles"
# testfiles source dirs for tools
SRC_H5LS_TESTFILES="$SRC_TOOLS_TESTFILES"
SRC_H5DUMP_TESTFILES="$SRC_TOOLS_TESTFILES"
SRC_H5DIFF_TESTFILES="$SRC_TOOLS/h5diff/testfiles"
SRC_H5COPY_TESTFILES="$SRC_TOOLS/h5copy/testfiles"
SRC_H5REPACK_TESTFILES="$SRC_TOOLS/h5repack/testfiles"
SRC_H5JAM_TESTFILES="$SRC_TOOLS/h5jam/testfiles"
SRC_H5STAT_TESTFILES="$SRC_TOOLS/h5stat/testfiles"
SRC_H5IMPORT_TESTFILES="$SRC_TOOLS/h5import/testfiles"

TEST_P_DIR=./testfiles
TESTDIR=./testfiles/xml
test -d $TEST_P_DIR || mkdir -p $TEST_P_DIR
test -d $TESTDIR || mkdir -p $TESTDIR

######################################################################
# test files
# --------------------------------------------------------------------
# All the test files copy from source directory to test directory
# NOTE: Keep this framework to add/remove test files.
#       Any test files from other tools can be used in this framework.
#       This list are also used for checking exist.
#       Comment '#' without space can be used.
# --------------------------------------------------------------------
LIST_HDF5_TEST_FILES_XML="
$SRC_H5DUMP_TESTFILES/tall.h5
$SRC_H5DUMP_TESTFILES/tarray1.h5
$SRC_H5DUMP_TESTFILES/tarray2.h5
$SRC_H5DUMP_TESTFILES/tarray3.h5
$SRC_H5DUMP_TESTFILES/tarray6.h5
$SRC_H5DUMP_TESTFILES/tarray7.h5
$SRC_H5DUMP_TESTFILES/tattr.h5
$SRC_H5DUMP_TESTFILES/tbitfields.h5
$SRC_H5DUMP_TESTFILES/tcompound.h5
$SRC_H5DUMP_TESTFILES/tcompound2.h5
$SRC_H5DUMP_TESTFILES/tcompound_complex.h5
$SRC_H5DUMP_TESTFILES/tdatareg.h5
$SRC_H5DUMP_TESTFILES/tdset.h5
$SRC_H5DUMP_TESTFILES/tdset2.h5
$SRC_H5DUMP_TESTFILES/tempty.h5
$SRC_H5DUMP_TESTFILES/tenum.h5
$SRC_H5DUMP_TESTFILES/textlink.h5
$SRC_H5DUMP_TESTFILES/tfpformat.h5
$SRC_H5DUMP_TESTFILES/tgroup.h5
$SRC_H5DUMP_TESTFILES/thlink.h5
$SRC_H5DUMP_TESTFILES/tloop.h5
$SRC_H5DUMP_TESTFILES/tloop2.h5
$SRC_H5DUMP_TESTFILES/tmany.h5
$SRC_H5DUMP_TESTFILES/tname-amp.h5
$SRC_H5DUMP_TESTFILES/tname-apos.h5
$SRC_H5DUMP_TESTFILES/tname-gt.h5
$SRC_H5DUMP_TESTFILES/tname-lt.h5
$SRC_H5DUMP_TESTFILES/tname-quot.h5
$SRC_H5DUMP_TESTFILES/tname-sp.h5
$SRC_H5DUMP_TESTFILES/tnamed_dtype_attr.h5
$SRC_H5DUMP_TESTFILES/tnestedcomp.h5
$SRC_H5DUMP_TESTFILES/tnodata.h5
$SRC_H5DUMP_TESTFILES/tobjref.h5
$SRC_H5DUMP_TESTFILES/topaque.h5
$SRC_H5DUMP_TESTFILES/torderattr.h5
$SRC_H5DUMP_TESTFILES/tref.h5
$SRC_H5DUMP_TESTFILES/tref-escapes.h5
$SRC_H5DUMP_TESTFILES/tref-escapes-at.h5
$SRC_H5DUMP_TESTFILES/tsaf.h5
$SRC_H5DUMP_TESTFILES/tslink.h5
$SRC_H5DUMP_TESTFILES/tstring.h5
$SRC_H5DUMP_TESTFILES/tstring-at.h5
$SRC_H5DUMP_TESTFILES/tstr.h5
$SRC_H5DUMP_TESTFILES/tstr2.h5
$SRC_H5DUMP_TESTFILES/tudlink.h5
$SRC_H5DUMP_TESTFILES/tvldtypes1.h5
$SRC_H5DUMP_TESTFILES/tvldtypes2.h5
$SRC_H5DUMP_TESTFILES/tvldtypes3.h5
$SRC_H5DUMP_TESTFILES/tvldtypes4.h5
$SRC_H5DUMP_TESTFILES/tvldtypes5.h5
$SRC_H5DUMP_TESTFILES/tvlstr.h5
"

LIST_OTHER_TEST_FILES_XML="
$SRC_H5DUMP_TESTFILES/tall.h5.xml
$SRC_H5DUMP_TESTFILES/tall-2A.h5.xml
$SRC_H5DUMP_TESTFILES/tarray1.h5.xml
$SRC_H5DUMP_TESTFILES/tarray2.h5.xml
$SRC_H5DUMP_TESTFILES/tarray3.h5.xml
$SRC_H5DUMP_TESTFILES/tarray6.h5.xml
$SRC_H5DUMP_TESTFILES/tarray7.h5.xml
$SRC_H5DUMP_TESTFILES/tattr.h5.xml
$SRC_H5DUMP_TESTFILES/tbitfields.h5.xml
$SRC_H5DUMP_TESTFILES/tcompound_complex.h5.xml
$SRC_H5DUMP_TESTFILES/tcompound.h5.xml
$SRC_H5DUMP_TESTFILES/tcompound2.h5.xml
$SRC_H5DUMP_TESTFILES/tdatareg.h5.xml
$SRC_H5DUMP_TESTFILES/tdset.h5.xml
$SRC_H5DUMP_TESTFILES/tdset2.h5.xml
$SRC_H5DUMP_TESTFILES/tempty.h5.xml
$SRC_H5DUMP_TESTFILES/tempty-dtd.h5.xml
$SRC_H5DUMP_TESTFILES/tempty-dtd-2.h5.xml
$SRC_H5DUMP_TESTFILES/tempty-dtd-uri.h5.xml
$SRC_H5DUMP_TESTFILES/tempty-nons.h5.xml
$SRC_H5DUMP_TESTFILES/tempty-nons-2.h5.xml
$SRC_H5DUMP_TESTFILES/tempty-nons-uri.h5.xml
$SRC_H5DUMP_TESTFILES/tempty-ns.h5.xml
$SRC_H5DUMP_TESTFILES/tempty-ns-2.h5.xml
$SRC_H5DUMP_TESTFILES/tenum.h5.xml
$SRC_H5DUMP_TESTFILES/textlink.h5.xml
$SRC_H5DUMP_TESTFILES/tfpformat.h5.xml
$SRC_H5DUMP_TESTFILES/tgroup.h5.xml
$SRC_H5DUMP_TESTFILES/thlink.h5.xml
$SRC_H5DUMP_TESTFILES/tloop.h5.xml
$SRC_H5DUMP_TESTFILES/tloop2.h5.xml
$SRC_H5DUMP_TESTFILES/tmany.h5.xml
$SRC_H5DUMP_TESTFILES/tname-amp.h5.xml
$SRC_H5DUMP_TESTFILES/tname-apos.h5.xml
$SRC_H5DUMP_TESTFILES/tnamed_dtype_attr.h5.xml
$SRC_H5DUMP_TESTFILES/tname-gt.h5.xml
$SRC_H5DUMP_TESTFILES/tname-lt.h5.xml
$SRC_H5DUMP_TESTFILES/tname-quot.h5.xml
$SRC_H5DUMP_TESTFILES/tname-sp.h5.xml
$SRC_H5DUMP_TESTFILES/tnestedcomp.h5.xml
$SRC_H5DUMP_TESTFILES/tnodata.h5.xml
$SRC_H5DUMP_TESTFILES/tobjref.h5.xml
$SRC_H5DUMP_TESTFILES/topaque.h5.xml
$SRC_H5DUMP_TESTFILES/torderattr1.h5.xml
$SRC_H5DUMP_TESTFILES/torderattr2.h5.xml
$SRC_H5DUMP_TESTFILES/torderattr3.h5.xml
$SRC_H5DUMP_TESTFILES/torderattr4.h5.xml
$SRC_H5DUMP_TESTFILES/tref.h5.xml
$SRC_H5DUMP_TESTFILES/tref-escapes.h5.xml
$SRC_H5DUMP_TESTFILES/tref-escapes-at.h5.xml
$SRC_H5DUMP_TESTFILES/tsaf.h5.xml
$SRC_H5DUMP_TESTFILES/tslink.h5.xml
$SRC_H5DUMP_TESTFILES/tstr.h5.xml
$SRC_H5DUMP_TESTFILES/tstr2.h5.xml
$SRC_H5DUMP_TESTFILES/tstring.h5.xml
$SRC_H5DUMP_TESTFILES/tstring-at.h5.xml
$SRC_H5DUMP_TESTFILES/tudlink.h5.xml
$SRC_H5DUMP_TESTFILES/tvldtypes1.h5.xml
$SRC_H5DUMP_TESTFILES/tvldtypes2.h5.xml
$SRC_H5DUMP_TESTFILES/tvldtypes3.h5.xml
$SRC_H5DUMP_TESTFILES/tvldtypes4.h5.xml
$SRC_H5DUMP_TESTFILES/tvldtypes5.h5.xml
$SRC_H5DUMP_TESTFILES/tvlstr.h5.xml
"

#
# copy test files and expected output files from source dirs to test dir
#
COPY_TESTFILES="$LIST_HDF5_TEST_FILES_XML $LIST_OTHER_TEST_FILES_XML"

COPY_TESTFILES_TO_TESTDIR()
{
    # copy test files. Used -f to make sure get a new copy
    for tstfile in $COPY_TESTFILES
    do
        # ignore '#' comment
        echo $tstfile | tr -d ' ' | grep '^#' > /dev/null
        RET=$?
        if [ $RET -eq 1 ]; then
            # skip cp if srcdir is same as destdir
            # this occurs when build/test performed in source dir and
            # make cp fail
            SDIR=`$DIRNAME $tstfile`
            INODE_SDIR=`$LS -i -d $SDIR | $AWK -F' ' '{print $1}'`
            INODE_DDIR=`$LS -i -d $TESTDIR | $AWK -F' ' '{print $1}'`
            if [ "$INODE_SDIR" != "$INODE_DDIR" ]; then
    	        $CP -f $tstfile $TESTDIR
                if [ $? -ne 0 ]; then
                    echo "Error: FAILED to copy $tstfile ."
                
                    # Comment out this to CREATE expected file
                    exit $EXIT_FAILURE
                fi
            fi
        fi
    done
}

CLEAN_TESTFILES_AND_TESTDIR()
{
    # skip rm if srcdir is same as destdir
    # this occurs when build/test performed in source dir and
    # make cp fail
    SDIR=`$DIRNAME $tstfile`
    INODE_SDIR=`$LS -i -d $SDIR | $AWK -F' ' '{print $1}'`
    INODE_DDIR=`$LS -i -d $TESTDIR | $AWK -F' ' '{print $1}'`
    if [ "$INODE_SDIR" != "$INODE_DDIR" ]; then
        $RM $TESTDIR
    fi
}

# Print a line-line message left justified in a field of 70 characters
# beginning with the word "Testing".
#
TESTING() {
   SPACES="                                                               "
   echo "Testing $* $SPACES" | cut -c1-70 | tr -d '\012'
}

# Source in the output filter function definitions.
. $srcdir/../../bin/output_filter.sh

# Run a test and print PASS or *FAIL*.  If a test fails then increment
# the `nerrors' global variable and (if $verbose is set) display the
# difference between the actual output and the expected output. The
# expected output is given as the first argument to this function and
# the actual output file is calculated by replacing the `.ddl' with
# `.out'.  The actual output is not removed if $HDF5_NOCLEANUP has a
# non-zero value.
#
TOOLTEST() {
   expect="$TESTDIR/$1"
   actual="$TESTDIR/`basename $1 .xml`.out"
   actual_err="$TESTDIR/`basename $1 .xml`.err"
   shift

    # Run test.
    TESTING $DUMPER $@
    (
    cd $TESTDIR
      $RUNSERIAL $DUMPER_BIN "$@"
    ) >$actual 2>$actual_err


   if [ ! -f $expect ]; then
    # Create the expect file if it doesn't yet exist.
    echo " CREATED"
     cp $actual $expect      
   elif  $CMP $expect $actual; then
      echo " PASSED"
   else
      echo "*FAILED*"
      echo "    Expected result (*.xml) differs from actual result (*.out)"
      nerrors="`expr $nerrors + 1`"
      test yes = "$verbose" && $DIFF $expect $actual |sed 's/^/    /'
   fi

   # Clean up output file
   if test -z "$HDF5_NOCLEANUP"; then
      rm -f $actual $actual_err
   fi
}

# Print a "SKIP" message    
SKIP() {
         TESTING $DUMPER $@
          echo  " -SKIP-"
}


##############################################################################
##############################################################################
###        T H E   T E S T S                                ###
##############################################################################
##############################################################################
# prepare for test
COPY_TESTFILES_TO_TESTDIR

# test XML
TOOLTEST tall.h5.xml --xml tall.h5
TOOLTEST tattr.h5.xml --xml tattr.h5
TOOLTEST tbitfields.h5.xml --xml tbitfields.h5
TOOLTEST tcompound.h5.xml --xml tcompound.h5
TOOLTEST tcompound2.h5.xml --xml tcompound2.h5
TOOLTEST tdatareg.h5.xml --xml tdatareg.h5
TOOLTEST tdset.h5.xml --xml tdset.h5
TOOLTEST tdset2.h5.xml --xml tdset2.h5
TOOLTEST tenum.h5.xml --xml tenum.h5
TOOLTEST tgroup.h5.xml --xml tgroup.h5
TOOLTEST thlink.h5.xml --xml thlink.h5
TOOLTEST tloop.h5.xml --xml tloop.h5
TOOLTEST tloop2.h5.xml --xml tloop2.h5
TOOLTEST tmany.h5.xml --xml tmany.h5
TOOLTEST tnestedcomp.h5.xml --xml tnestedcomp.h5
TOOLTEST tcompound_complex.h5.xml --xml tcompound_complex.h5
TOOLTEST tobjref.h5.xml --xml tobjref.h5
TOOLTEST topaque.h5.xml --xml topaque.h5
TOOLTEST tslink.h5.xml --xml tslink.h5
TOOLTEST tudlink.h5.xml --xml tudlink.h5
TOOLTEST textlink.h5.xml --xml textlink.h5
TOOLTEST tstr.h5.xml --xml tstr.h5
TOOLTEST tstr2.h5.xml --xml tstr2.h5
TOOLTEST tref.h5.xml --xml tref.h5
TOOLTEST tname-amp.h5.xml --xml tname-amp.h5
TOOLTEST tname-apos.h5.xml --xml tname-apos.h5
TOOLTEST tname-gt.h5.xml --xml tname-gt.h5
TOOLTEST tname-lt.h5.xml --xml tname-lt.h5
TOOLTEST tname-quot.h5.xml --xml tname-quot.h5
TOOLTEST tname-sp.h5.xml --xml tname-sp.h5
TOOLTEST tstring.h5.xml --xml tstring.h5
TOOLTEST tstring-at.h5.xml --xml tstring-at.h5
TOOLTEST tref-escapes.h5.xml --xml tref-escapes.h5
TOOLTEST tref-escapes-at.h5.xml --xml tref-escapes-at.h5
TOOLTEST tnodata.h5.xml --xml tnodata.h5
TOOLTEST tarray1.h5.xml --xml tarray1.h5
TOOLTEST tarray2.h5.xml --xml tarray2.h5
TOOLTEST tarray3.h5.xml --xml tarray3.h5
TOOLTEST tarray6.h5.xml --xml tarray6.h5
TOOLTEST tarray7.h5.xml --xml tarray7.h5
TOOLTEST tvldtypes1.h5.xml --xml tvldtypes1.h5
TOOLTEST tvldtypes2.h5.xml --xml tvldtypes2.h5
TOOLTEST tvldtypes3.h5.xml --xml tvldtypes3.h5
TOOLTEST tvldtypes4.h5.xml --xml tvldtypes4.h5
TOOLTEST tvldtypes5.h5.xml --xml tvldtypes5.h5
TOOLTEST tvlstr.h5.xml --xml tvlstr.h5
TOOLTEST tsaf.h5.xml --xml tsaf.h5
TOOLTEST tempty.h5.xml --xml tempty.h5
TOOLTEST tnamed_dtype_attr.h5.xml --xml tnamed_dtype_attr.h5
##Test dataset and attribute of null space.  Commented out:
## wait until the XML schema is updated for null space. 
##TOOLTEST tnullspace.h5.xml --xml tnulspace.h5

# other options for xml

TOOLTEST tempty-dtd.h5.xml --xml --use-dtd tempty.h5
TOOLTEST tempty-dtd-2.h5.xml --xml -u tempty.h5

# The lone colon here confuses some systems (Cray X1).  Skip
# it if configure detects that this is a problem.
if test "X$H5_LONE_COLON" != "Xno"; then 
  TOOLTEST tempty-nons.h5.xml --xml -X ":" tempty.h5
else
  SKIP tempty-nons.h5.xml --xml -X ":" tempty.h5
fi

TOOLTEST tempty-nons-2.h5.xml --xml --xml-ns=":" tempty.h5

## Some of these combinations are syntactically correct but
##  the URLs are dummies 
TOOLTEST tempty-ns.h5.xml --xml -X "thing:" tempty.h5
TOOLTEST tempty-ns-2.h5.xml --xml --xml-ns="thing:" tempty.h5
TOOLTEST tempty-nons-uri.h5.xml --xml --xml-ns=":" --xml-dtd="http://somewhere.net" tempty.h5
TOOLTEST tempty-dtd-uri.h5.xml --xml --use-dtd --xml-dtd="http://somewhere.net" tempty.h5

TOOLTEST tall-2A.h5.xml --xml -A tall.h5


# tests for attribute order
TOOLTEST torderattr1.h5.xml --xml -H --sort_by=name --sort_order=ascending torderattr.h5
TOOLTEST torderattr2.h5.xml --xml -H --sort_by=name --sort_order=descending torderattr.h5
TOOLTEST torderattr3.h5.xml --xml -H --sort_by=creation_order --sort_order=ascending torderattr.h5
TOOLTEST torderattr4.h5.xml --xml -H --sort_by=creation_order --sort_order=descending torderattr.h5

# tests for floating point user defined printf format
TOOLTEST tfpformat.h5.xml -u -m %.7f tfpformat.h5

# Clean up temporary files/directories
CLEAN_TESTFILES_AND_TESTDIR

if test $nerrors -eq 0 ; then
    echo "All $TESTNAME tests passed."
    exit $EXIT_SUCCESS
else
    echo "$TESTNAME tests failed with $nerrors errors."
    exit $EXIT_FAILURE
fi
